use bevy::prelude::*;
use std::time::Duration;

use bevy::utils::HashMap;

use std::hash::Hash;

#[derive(Bundle)]
pub struct AnimBundle<Tag: Hash + Eq + Component> {
    pub tag: Tag,
    pub mgr: AnimationManager<Tag>,
}

#[derive(Default, Component)]
pub(crate) struct AnimationManager<Tag: Hash + Eq> {
    animations: HashMap<Tag, Animation>,
}

impl<Tag: Hash + Eq> AnimationManager<Tag> {
    pub fn insert(mut self, which: Tag, anim: Animation) -> Self {
        self.animations.insert(which, anim);
        self
    }
}

pub(crate) struct Animation {
    atlas: TextureAtlas,
    max_index: usize,
    fps: u8,
    frame_timer: Timer,
    tex: Handle<Image>,
}

impl Animation {
    pub fn new(atlas: TextureAtlas, length: usize, fps: u8, tex_handle: Handle<Image>) -> Self {
        Self {
            atlas,
            max_index: length - 1,
            fps,
            frame_timer: Self::timer_from_fps(fps),
            tex: tex_handle,
        }
    }

    // stolen from https://github.com/bevyengine/bevy/blob/5c759a1be800209f537bea31d32b8ba7e966b0c1/examples/2d/sprite_animation.rs#L53-L55
    pub(crate) fn timer_from_fps(fps: u8) -> Timer {
        Timer::new(Duration::from_secs_f32(1.0 / (fps as f32)), TimerMode::Once)
    }
}
